package cn.com.infosec.crypto.encodings;

import cn.com.infosec.crypto.AsymmetricBlockCipher;
import cn.com.infosec.crypto.CipherParameters;
import cn.com.infosec.crypto.InvalidCipherTextException;
import cn.com.infosec.crypto.params.AsymmetricKeyParameter;
import cn.com.infosec.crypto.params.ParametersWithRandom;
import cn.com.infosec.jce.logger.ConsoleLogger;
import java.security.AccessController;
import java.security.SecureRandom;

public class PKCS1Encoding implements AsymmetricBlockCipher {
   public static final String STRICT_LENGTH_ENABLED_PROPERTY = "cn.com.infosec.pkcs1.strict";
   private static final int HEADER_LENGTH = 10;
   private SecureRandom random;
   private AsymmetricBlockCipher engine;
   private boolean forEncryption;
   private boolean forPrivateKey;
   private boolean useStrictLength;

   public PKCS1Encoding(AsymmetricBlockCipher cipher) {
      this.engine = cipher;
      this.useStrictLength = this.useStrict();
   }

   private boolean useStrict() {
      String var1;
      return (var1 = (String)AccessController.doPrivileged(new PKCS1Encoding$1(this))) == null || var1.equals("true");
   }

   private byte[] encodeBlock(byte[] in, int inOff, int inLen) {
      int var4;
      byte[] var5 = new byte[var4 = this.engine.getInputBlockSize()];
      int var6;
      if (this.forPrivateKey) {
         var5[0] = 1;

         for(var6 = 1; var6 != var4 - inLen - 1; ++var6) {
            var5[var6] = -1;
         }
      } else {
         this.random.nextBytes(var5);
         var5[0] = 2;

         for(var6 = 1; var6 != var4 - inLen - 1; ++var6) {
            while(var5[var6] == 0) {
               var5[var6] = (byte)this.random.nextInt();
            }
         }
      }

      int var7;
      var5[(var7 = var4 - inLen) - 1] = 0;
      System.arraycopy(in, inOff, var5, var7, inLen);
      return this.engine.processBlock(var5, 0, var4);
   }

   private byte[] decodeBlock(byte[] in, int inOff, int inLen) {
      byte[] var10000 = in = this.engine.processBlock(in, inOff, inLen);
      ConsoleLogger.logBinary("block", in);
      if (var10000.length < this.getOutputBlockSize()) {
         throw new InvalidCipherTextException("block truncated");
      } else {
         byte inOff1;
         if ((inOff1 = in[0]) != 1 && inOff1 != 2) {
            throw new InvalidCipherTextException("unknown block type");
         } else if (this.useStrictLength && in.length != this.engine.getOutputBlockSize()) {
            throw new InvalidCipherTextException("block incorrect size");
         } else {
            int var4;
            byte inLen1;
            for(var4 = 1; var4 != in.length && (inLen1 = in[var4]) != 0; ++var4) {
               if (inOff1 == 1 && inLen1 != -1) {
                  throw new InvalidCipherTextException("block padding incorrect");
               }
            }

            if (++var4 < in.length && var4 >= 10) {
               byte[] inLen2;
               var10000 = inLen2 = new byte[inOff = in.length - var4];
               System.arraycopy(in, var4, inLen2, 0, inOff);
               ConsoleLogger.logBinary("result", inLen2);
               return var10000;
            } else {
               throw new InvalidCipherTextException("no data in block");
            }
         }
      }
   }

   public AsymmetricBlockCipher getUnderlyingCipher() {
      return this.engine;
   }

   public void init(boolean forEncryption, CipherParameters param) {
      AsymmetricKeyParameter var4;
      if (param instanceof ParametersWithRandom) {
         ParametersWithRandom var3;
         this.random = (var3 = (ParametersWithRandom)param).getRandom();
         var4 = (AsymmetricKeyParameter)var3.getParameters();
      } else {
         SecureRandom var5;
         var5 = new SecureRandom.<init>();
         this.random = var5;
         var4 = (AsymmetricKeyParameter)param;
      }

      this.engine.init(forEncryption, param);
      this.forPrivateKey = var4.isPrivate();
      this.forEncryption = forEncryption;
   }

   public int getInputBlockSize() {
      PKCS1Encoding var10000 = this;
      int var1 = this.engine.getInputBlockSize();
      return var10000.forEncryption ? var1 - 10 : var1;
   }

   public int getOutputBlockSize() {
      PKCS1Encoding var10000 = this;
      int var1 = this.engine.getOutputBlockSize();
      return var10000.forEncryption ? var1 : var1 - 10;
   }

   public byte[] processBlock(byte[] in, int inOff, int inLen) {
      return this.forEncryption ? this.encodeBlock(in, inOff, inLen) : this.decodeBlock(in, inOff, inLen);
   }
}
